package de.dhbw.contactdb;

import java.time.LocalDateTime;


/**
 * Main class
 *
 * @author florian kaiser
 * @version 1.0
 * @since 21.06.2021
 */
public class Visit {
    /**
     * place where person is
     */
    private final Place place;
    /**
     * start date of visit
     */
    private LocalDateTime start;
    /**
     * end date of visit
     */
    private LocalDateTime end;
    /**
     * person who visits
     */
    private Person person;


    /**
     * Constructor
     *
     * @param start  start date of visit
     * @param end    end date of visit
     * @param person person who visits
     * @param place  place where person is
     */
    public Visit(LocalDateTime start, LocalDateTime end, Person person, Place place) {
        this.start = start;
        this.end = end;
        this.person = person;
        this.place = place;
    }


    /**
     * @return {@link #start}
     */
    public LocalDateTime getStart() {
        return start;
    }


    /**
     * @param start {@link #start}
     */
    public void setStart(LocalDateTime start) {
        this.start = start;
    }


    /**
     * @return {@link #end}
     */
    public LocalDateTime getEnd() {
        return end;
    }


    /**
     * @param end {@link #end}
     */
    public void setEnd(LocalDateTime end) {
        this.end = end;
    }


    /**
     * @return {@link #person}
     */
    public Person getPerson() {
        return person;
    }


    /**
     * @param person {@link #person}
     */
    public void setPerson(Person person) {
        this.person = person;
    }


    /**
     * @return {@link #place}
     */
    public Place getPlace() {
        return place;
    }


    /**
     * @param start Start of timespan
     * @param end   End of timespan
     * @return boolean if timespan between start and end is overlapping timespan between this visits start and end
     */
    public boolean isOverlapping(LocalDateTime start, LocalDateTime end) {
        return ((null == end) || this.start.isBefore(end)) && ((null == this.end) || start.isBefore(this.end));
    }


    /**
     * @param inputTime time
     * @return boolean if time is between this visits start and end
     */
    public boolean isInTimePeriod(LocalDateTime inputTime) {
        return this.start.isBefore(inputTime) && this.end.isAfter(inputTime) || this.end.equals(inputTime) || this.start.equals(inputTime);
    }

    @Override
    public String toString() {
        return "Besuch{" +
                "start=" + start +
                ", end=" + end +
                ", person=" + person +
                ", ort=" + place +
                '}';
    }
}
