package de.dhbw.unit;

import de.dhbw.contactdb.Main;
import de.dhbw.contactdb.utils.Parser;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.util.stream.Collectors;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

/**
 * Tests
 */
public class ProjectTest {
    /**
     * Parse database before testing
     */
    @BeforeAll
    public static void before() {
        Parser.parse("/de/dhbw/contactdb/db/contacts2021.db");
    }


    /**
     * Test #1
     */
    @Test
    public void testSearchPerson() {
        assertEquals("Mila Milan ", Main.searchForPerson("ila"));
    }


    /**
     * Test #2
     */
    @Test
    public void testSearchPlace() {
        assertEquals("Supermarkt GroÃŸmarkt ", Main.searchForPlace("markt"));
    }


    /**
     * Test #3
     */
    @Test
    public void testContactPersons() {
        assertEquals("Ben, Carla, Emilia, Emily, Joshua, Malia, Maria, Mia, Sophia", Main.getContactPersons(158).stream()
                .filter(visit -> visit.getPerson().getId() != 158)
                .map(visit -> visit.getPerson().getName())
                .sorted()
                .distinct()
                .collect(Collectors.joining(", ")));
    }


    /**
     * Test #4
     */
    @Test
    public void testVisitors() {
        assertEquals("Anna, Charlotte, Emilia, Leonie, Marie, Mia", Main.getVisitors("3,\"2021-05-15t11:00:00\"").stream()
                .map(visit -> visit.getPerson().getName())
                .sorted()
                .distinct()
                .collect(Collectors.joining(", ")));
    }


    /**
     * Test #5
     */
    @Test
    public void testImport() {
        assertNotNull(Main.class.getResourceAsStream("/de/dhbw/contactdb/db/contacts2021.db"));
    }


    /**
     * Test #6
     */
    @Test
    public void testGetSearchString() {
        assertEquals("3,2021-05-15t11:00:00", Main.getSearchString("=3,\"2021-05-15T11:00:00\""));
        assertEquals("3", Main.getSearchString("=3"));
        assertEquals("3", Main.getSearchString("=\"3\""));
        assertEquals("ila", Main.getSearchString("=ila"));
        assertEquals("ila", Main.getSearchString("=\"ila\""));
    }
}